/*
Programmer: Zach Hixson
Date: 9/13/2015
Purpose: Initializes Othello Program
*/

/*
TODO:
	-

BUGS:
	-
*/

/*########################
Declare Board Settings
########################*/
var boardColors = ["#00A704", "#00A704", "green"]; //[color1, color2, highlighted color]

//declare global variables
var canvas; //canvas object used for drawing
var c; //drawing context
var board = []; //array used to remember the state of each space on the board
var cellWidth; //determines width of each cell based on canvas width
var cellHeight; //determines height of each cell based on canvas height
var playerTurn = 2; //remembers the current turn; 0 = game over, 1 = white, 2 = black
var gameState = 0; //stores gamestate; 0 = not started; 1 = playing; 2 = game over
var AI;
var iUI;
var mouse;

//create board space class
class boardSpace{
	constructor(_x, _y){
		this.name = _x + "," + _y;
		this.piece = 0; //0-none, 1-white, 2-black
		this.x = _x;
		this.y = _y;
	}
}

window.onload = function(){
	canvas = document.getElementById("gameBoard");
	canvas.width = window.innerHeight * 0.95;
	canvas.height = window.innerHeight * 0.95;
	c = canvas.getContext("2d");
	cellWidth = Math.round(canvas.width / 8);
	cellHeight = Math.round(canvas.height / 8);
	
	//initate iUI
	iUI = new iUI_init(canvas);

	//fill in board array
	for (var x = 0; x < 8; x++){ //loops through first demension
		board.push([]);
		for (var y = 0; y < 8; y++){
			board[x].push(new boardSpace(x, y));
		}
	}
	
	//declare AI
	AI = new AI_init();
	
	//declare mouse
	mouse = new Mouse();
	window.onmousemove = mouse.updateMouse.bind(mouse);
	document.onmousedown = mouse.onMouseDown.bind(mouse);
	document.onmouseup = mouse.onMouseUp.bind(mouse);
	
	//setup center pieces to start game
	board[3][3].piece = 1;
	board[4][4].piece = 1;
	board[3][4].piece = 2;
	board[4][3].piece = 2;
	
	//debug pieces
	/*board[4][2].piece = 2;
	board[4][4].piece = 2;
	board[3][3].piece = 1;
	board[3][4].piece = 1;*/
	
	//starts main program
	AI.checkPossible(playerTurn);
	main();
}

function main(){
	switch (gameState){
		case 0:
			drawStart();
			break;
		case 1:
			drawBoard();
			drawPieces();
			drawUI();
			break;
		case 2:
			drawBoard();
			drawPieces();
			drawEnd();
	}
}
/*
Handles Drawing events
*/

//debug variables
var debugWorkingSpaces = false;

function drawStart(){
	//draw background
	c.fillStyle = boardColors[0];
	c.fillRect(0,0,canvas.width, canvas.height);
	
	c.strokeStyle = "black";
	c.strokeRect(0,0,canvas.width, canvas.height);
	
	//draw Title
	c.font = canvas.width/5 + "px sans-serif"
	c.fillStyle = "white";
	c.fillText("Othello", canvas.width/6.2, canvas.height / 2.12);
	
	c.fillStyle = "black";
	c.fillText("Othello", canvas.width/6, canvas.height / 2.1);
	
	//draw menu box
	c.shadowBlur=20;
	c.shadowColor="black";
	c.fillStyle = "#00D704";
	c.fillRect(
		canvas.width/4,
		canvas.height / 1.9,
		canvas.width/2,
		canvas.height / 3.8);
	c.strokeStyle = "black";
	c.strokeRect(
		canvas.width/4,
		canvas.height / 1.9,
		canvas.width/2,
		canvas.height / 3.8);
	c.shadowBlur = 0;
	
	//draw buttons
	iUI.addText("txtSinglePlayer", "Two Player", 8, 80,115);
	iUI.addButton("btn_2player", "Start", 93,119,14,10);
	iUI.obj.btn_2player.onClick = function(){
		gameState = 1;
		AI.difficulty = 0;
		drawStart();
	}
	
	iUI.addText("txtAI", "Computer", 8, 83,138);
	iUI.addButton("btn_Easy", "Easy", 71,142,14.5,10);
	iUI.obj.btn_Easy.onClick = function(){
		gameState = 1;
		AI.difficulty = 1;
		drawStart();
	}
	
	iUI.addButton("btn_Medium", "Medium", 89,142,23,10);
	iUI.obj.btn_Medium.onClick = function(){
		gameState = 1;
		AI.difficulty = 2;
		drawStart();
	}
	
	iUI.addButton("btn_Hard", "Hard", 115,142,14,10);
	iUI.obj.btn_Hard.onClick = function(){
		gameState = 1;
		AI.difficulty = 3;
		drawStart();
	}
	
	//adds all menu items to a group
	iUI.addGroup("grpMainBtns");
	iUI.obj.grpMainBtns.addItems(
		"btn_2player",
		"btn_Easy",
		"btn_Medium",
		"btn_Hard",
		"txtSinglePlayer",
		"txtAI"
		)
	
	if (gameState != 0){
		iUI.obj.grpMainBtns.changeProp("enabled", false);
	}
	
	iUI.drawUI();
}

function drawEnd(){
	var winner;
	
	//draw menu box
	c.shadowBlur=20;
	c.shadowColor="black";
	c.fillStyle = "#00D704";
	c.fillRect(
		canvas.width/4,
		canvas.height / 3,
		canvas.width/2,
		canvas.height / 3.1);
	c.strokeStyle = "black";
	c.shadowBlur = 0;

	if (AI.difficulty == 0){
		if (AI.score[0] != AI.score[1]){ //check to see if the game is not tied
			//draw text
			iUI.addText("txtWinner", "Winner:", 15, 53,90);
			
			//figure out who won
			if (AI.score[0] > AI.score[1]){
				winner = "White";
			}
			else if (AI.score[1] > AI.score[0]){
				winner = "Black";
			}
			
			iUI.addText("txtActualWinner", winner, 15, 108,90);
			iUI.obj.txtActualWinner.color = winner;
			
			iUI.addText("txtWhiteScore", AI.score[0], 8, 80,103);
			iUI.obj.txtWhiteScore.color = "white";
			
			iUI.addText("txtTo", "to", 8, 98,103);
			
			iUI.addText("txtBlackScore", AI.score[1], 8, 113,103);
			iUI.obj.txtBlackScore.color = "Black";
		}
		else{ //if the game is tied, change display
			iUI.addText("txtTied", "Tie!", 15, 88,90);
			
			iUI.addText("txtWhiteScore", AI.score[0], 8, 80,103);
			iUI.obj.txtWhiteScore.color = "white";
			
			iUI.addText("txtTo", "to", 8, 98,103);
			
			iUI.addText("txtBlackScore", AI.score[1], 8, 113,103);
			iUI.obj.txtBlackScore.color = "Black";
		}
	}
	else{ //if you are playing against the ai...
		//figure out difficulty
		var difficulty;
		
		switch(AI.difficulty){
			case 1:
				difficulty = "Easy";
				break;
			case 2:
				difficulty = "Medium";
				break;
			case 3:
				difficulty = "Hard";
				break;
		}
		
		//figure out who won
		if (AI.score[0] > AI.score[1]){
			winner = 1;
		}
		else if (AI.score[1] > AI.score[0]){
			winner = 2;
		}
		else{
			winner = 2;
		}
		
		if (winner == AI.color){
			iUI.addText("txtAIWin", "You Lost!", 15, 70,90);
		}
		else if (winner != AI.color && winner != 3){
			iUI.addText("txtYouWin", "You Won!", 15, 70,90);
		}
		else{
			iUI.addText("txtTied", "Tie!", 15, 88,90);
		}
		
		iUI.addText("txtDifficulty", "Difficulty: " + difficulty, 4, 53,73);
		
		iUI.addText("txtWhiteScore", AI.score[0], 8, 80,103);
		iUI.obj.txtWhiteScore.color = "white";
		
		iUI.addText("txtTo", "to", 8, 98,103);
		
		iUI.addText("txtBlackScore", AI.score[1], 8, 113,103);
		iUI.obj.txtBlackScore.color = "Black";
	}
	
	iUI.addButton("btnMainMenu", "Main Menu", 85, 113, 32, 10)
	iUI.obj.btnMainMenu.onClick = function(){
		location.reload();
	}
	
	iUI.drawUI();
}

function drawBoard(){
	//draw background
	c.fillStyle = boardColors[0];
	c.fillRect(0,0,canvas.width, canvas.height);
	
	//draw lines
	c.strokeStyle = "black";
	c.lineWidth = 1;
	
	for (var rowCol = 0; rowCol < 8; rowCol ++){
		//draw row
		c.beginPath();
		c.moveTo(0, (rowCol * cellHeight) + 0.5);
		c.lineTo(canvas.width, (rowCol * cellHeight) + 0.5);
		c.stroke();
		
		//draw column
		c.beginPath();
		c.moveTo((rowCol * cellWidth) + 0.5, 0);
		c.lineTo((rowCol * cellWidth) + 0.5, canvas.height);
		c.stroke();
	}
	
	//draw outer border
	c.strokeStyle = "black";
	c.strokeRect(0,0,canvas.width, canvas.height);
}

function drawPieces(){
	var x;
	var y;
	
	//fill in pieces
	for (var row = 0; row < 8; row++){ //iterate through rows
		for (var col = 0; col < 8; col++){ //iterate through columns
			//check to see if there is a piece in that space
			if (board[row][col].piece != 0){
				switch(board[row][col].piece){
					case 1:
						c.fillStyle = "white";
						break;
					case 2:
						c.fillStyle = "black";
						break;
				}
				
				//draw piece
				x = (row * cellWidth) + (cellWidth/2);
				y = (col * cellHeight) + (cellHeight/2);
				
				fillCircle(x, y, (cellWidth / 2) * 0.75);
			}
		}
	}
	
	//debug working spaces
	if (debugWorkingSpaces){
		for (var i = 0; i < AI.workingSpaces.length; i++){
			x = (AI.workingSpaces[i][1] * cellWidth) + (cellWidth/2);
			y = (AI.workingSpaces[i][2] * cellHeight) + (cellHeight/2);
			
			c.fillStyle = "red";
			fillCircle(x, y, (cellWidth / 2) * 0.1);
		}
	}
}

function drawUI(){
	//if mouse is  within bounds
	if (mouseInBounds()){
		//draw cursor rectangle
		c.fillStyle = "rgba(104,255,235,0.5)";
		c.fillRect(mouse.cellX * cellWidth, mouse.cellY * cellHeight, cellWidth, cellHeight);
		
		if (AI.difficulty == 0 || playerTurn != AI.color){
			//draw piece to be placed
			if (board[mouse.cellX][mouse.cellY].piece == 0){
				x = (mouse.cellX * cellWidth) + (cellWidth/2);
				y = (mouse.cellY * cellHeight) + (cellHeight/2);
				
				switch(playerTurn){
					case 1:
						c.fillStyle = "white";
						break;
					case 2:
						c.fillStyle = "black";
						break;
				}
				fillCircle(x, y, (cellWidth / 2) * 0.60);
			}
		}
	}
}

function fillCircle(x, y, r){
	c.beginPath();
	c.arc(x, y, r, 0, Math.PI * 2, false); //draws arc (x, y, radius, start arc, end arc, reverse start/end)
	c.fill();
}
/*
Handles Event Functions
*/

/*##################
					handles mouse events
##################*/

class Mouse{
	constructor(){
		this.x = -1;
		this.y = -1;
		this.offsetX = 0;
		this.offsetY = 0;
		this.bounds;
		this.cellX = 0;
		this.cellY = 0;
		this.mouseDown = false;
	}
	
	updateMouse(event){
		this.bounds = canvas.getBoundingClientRect();
		this.offsetX = this.bounds.left;
		this.offsetY = this.bounds.top;
		
		//get mouse position
		this.x = event.clientX - this.offsetX;
		this.y = event.clientY - this.offsetY;
		
		//get cell that the mouse is hovering over
		mouse.cellX = Math.max(Math.min(Math.floor(this.x / (canvas.width/8)), 7), 0);
		mouse.cellY = Math.max(Math.min(Math.floor(this.y / (canvas.height/8)), 7), 0);
		
		//integrate with iUI library
		iUI.mouse.x = this.x;
		iUI.mouse.y = this.y;
		
		main();
	}
	
	onMouseDown(){
		this.mouseDown = true;
		iUI.mouse.mouseDown = true;
		
		if (gameState == 1){
			placePiece(this.cellX, this.cellY);
		}
		
		main();
	}
	
	onMouseUp(){
		this.mouseDown = false;
		iUI.mouse.mouseDown = false;
		
		main();
	}
}

/*##################
					handles main logic
##################*/

function placePiece(_cellX, _cellY){
	if (AI.difficulty == 0 || playerTurn != AI.color){ //make sure it isn't the AI's turn
	//if the space is empty, place piece
		if (mouseInBounds()){
			if (board[mouse.cellX][mouse.cellY].piece == 0){ //check if space is empty
				//get list of info if space is possible (_cellX, _cellY, terminalXYs, _playerTurn, flipped)
				var spaceInfo = tryflip(_cellX, _cellY, playerTurn);
				
				if (spaceInfo[2].length > 0){ //if possible, flip pieces, if not return false
					
					//flip pieces
					flipPieces(_cellX, _cellY, spaceInfo[2], playerTurn);
					
					//switch the player turn
					switchTurns();
					
					AI.checkPossible(playerTurn);
					
					if (AI.difficulty > 0 && playerTurn == AI.color && gameState == 1){ //check to see if AI is enabled
						AI.startTurn();
					}
				}
			}
		}
	}
	
	//update board
	main();
}

function tryflip(_cellX, _cellY, _playerTurn){ //flip all possible pieces, if no flips are possible, return false
	//declare variables
	var stillSearching = true;
	var canCapture = false; //variable changes based on if the player can completed a 'trap'
	var isValid = false; //will be returned if it is a valid move or not (must be at least 1 capture or "trap")
	var checkX;
	var checkY;
	var uD = 1; //up, down; Will be added to the loop (set to 1 or -1)
	var lR = 1; //left, right; Will be added to the loop (set to 1 or -1)
	var terminalX; //the X of the terminal piece
	var terminalY; //the Y of the terminal piece
	var terminalXYs = [];
	var flipped = 0;
	
	for (var i = 0; i < 8; i++){
		canCapture = false; //reset canCapture
		stillSearching = true; //reset stillSearching
		
		switch (i){
			case 0: //up
				uD = -1;
				lR = 0;
				break;
			case 1: //down
				uD = 1;
				lR = 0;
				break;
			case 2: //left
				uD = 0;
				lR = -1;
				break;
			case 3: //right
				uD = 0;
				lR = 1;
				break;
			case 4: //^<
				uD = -1;
				lR = -1;
				break;
			case 5: //^>
				uD = -1;
				lR = 1;
				break;
			case 6: //v<
				uD = 1;
				lR = - 1;
				break;
			case 7: //v>
				uD = 1;
				lR = 1;
				break;
		}
		
		//offset search by 1 cell in given direction (keeps it from "finding" itself)
		checkY = _cellY + uD;
		checkX = _cellX + lR;
		
		//make sure checkY and checkX are within bounds
		checkY = Math.max(Math.min(checkY, 7), 0);
		checkX = Math.max(Math.min(checkX, 7), 0);
		
		//dertermine if pieces can be trapped, and if so trap them
		while (stillSearching){
			if (board[checkX][checkY].piece == _playerTurn
				&& board[checkX][checkY].piece != 0
				&& canCapture == false
				&& (Math.abs(_cellY - checkY) >= 2 || Math.abs(_cellX - checkX) >= 2)){
				
				canCapture = true;
				isValid = true;
				terminalX = checkX;
				terminalY = checkY;
				
				flipped += (Math.sqrt(Math.pow(_cellX - terminalX,2) + Math.pow(_cellY - terminalY,2)))
				terminalXYs.push([terminalX, terminalY, uD, lR]);
			}
			else if (board[checkX][checkY].piece == 0 && !canCapture){ //if it reached an empty space, stop
				stillSearching = false;
			}
			else if (board[checkX][checkY].piece == _playerTurn){ //if the piece right next to it is the same, stop
				stillSearching = false;
			}
			
			//move to next cell in the given direction
			checkY += uD;
			checkX += lR;
			
			if (!isInBounds(checkY, 0, 7) || !isInBounds(checkX, 0, 7) || canCapture){
				stillSearching = false;
			}
		}
	}
	
	flipped -= terminalXYs.length
	
	return [_cellX, _cellY, terminalXYs, _playerTurn, flipped];
}

function flipPieces(_cellX, _cellY, _termXYs, _playerTurn){
	for (var i in _termXYs){
		var stillFlipping = true;
				
		//first invert the search (starting with terminal, ending with starting piece)
		var uD = _termXYs[i][2] * -1;
		var lR = _termXYs[i][3] * -1;
		
		var checkX = _termXYs[i][0] + lR;
		var checkY = _termXYs[i][1] + uD;
		
		while (stillFlipping){
			if (checkX != _cellX || checkY != _cellY){
				board[checkX][checkY].piece = _playerTurn;
				main();
				checkY += uD;
				checkX += lR;
			}
			else{
				stillFlipping = false;
			}
		}
	}
		
	//place a piece in the highlighted cell
	board[_cellX][_cellY].piece = playerTurn;
}

function switchTurns(){
	switch(playerTurn){
		case 1:
			playerTurn = 2;
			break;
		case 2:
			playerTurn = 1;
			break;
	}
}

/*##################
					handles secondary logic
##################*/

function mouseInBounds(){
	if ((mouse.x >= 0 && mouse.x <= canvas.width) && (mouse.y >= 0 && mouse.y <= canvas.height)){
		return true;
	}
	else{
		return false;
	}
}

function isInBounds(value, min, max){
	if (value >= min && value <= max){
		return true;
	}
	else{
		return false;
	}
}
/*
Contains all AI Code
*/

class AI_init { //Main iUI class
	constructor(){
		this.difficulty = 0; //0 = off; 1 = easy, 2 = med, 3 = hard; off is used to just check the board status.
		this.possibleSpaces = []; //stores all possible spaces for next turn. Used by both AI and general game
		this.workingSpaces = [];
		this.color = 1; //sets the color (and by extention turn) of the AI
		this.score = [0,0];
		this.debug = false;
		this.timerLength = 0;
		this.timer; //a timer to use so it doesn't take turns immediately
		this.timerLoop;
	}
	
	startTurn(){
		this.timer = this.timerLength;
		this.timerLoop = setInterval('AI.tickTimer()', 1000);
	}
	
	tickTimer(){
		if (this.timer > 0){
			this.timer -= 1;
			console.log(this.timer)
		}
		else{
			this.takeTurn();
			switchTurns();
			this.checkPossible(playerTurn);
			main();
			
			//reset timer
			clearInterval(this.timerLoop);
		}
	}
	
	logPossible(){
		console.log("Length: " + this.possibleSpaces.length);
		for (var i in this.possibleSpaces){
			console.log(this.possibleSpaces[i][0] + "," + this.possibleSpaces[i][1]);
		}
	}
	
	checkPossible(_playerTurn){ //checks all possible places for next move (if none then it ends game)
		this.possibleSpaces = []; //reset possibleSpaces
		var checkSpace = [0,0];
		var opositePlayer = 0;
		var isPossible = false;
		
		//figure out which player to check for (we want to check for the oposite pieces)
		switch(_playerTurn){
			case 1:
				opositePlayer = 2;
				break;
			case 2:
				opositePlayer = 1;
				break;
		}
		
		//iterate through board spaces using gaussian check
		for (var x = 0; x < 8; x++){
			for (var y = 0; y < 8; y++){
				isPossible = false; //reset isPossible for every space
				
				if (board[x][y].piece == 0){ //check to make sure there is no piece already there
					for (var i = 0; i < 8; i++){ //loop through remaining 8 spaces of gaussian square
						switch (i){
							case 0:
								checkSpace = [-1,-1];
								break;
							case 1:
								checkSpace = [0,-1];
								break;
							case 2:
								checkSpace = [1,-1];
								break;
							case 3:
								checkSpace = [-1,0];
								break;
							case 4:
								checkSpace = [1,0];
								break;
							case 5:
								checkSpace = [-1,1];
								break;
							case 6:
								checkSpace = [0,1];
								break;
							case 7:
								checkSpace = [1,1];
								break;
						}
						
						if (isInBounds(x + checkSpace[0], 0, 7) && isInBounds(y + checkSpace[1], 0, 7)){
							if (board[x + checkSpace[0]][y + checkSpace[1]].piece == opositePlayer && !isPossible){
								//add space to possible spaces list
								this.possibleSpaces.push([x, y])
								isPossible = true;
							}
						}
					}
				}
			}
		}
		
		//loop through possible spaces, and out of those check to see which ones are legitimate (and their #flipped)
		var currentSpace = [];
		this.workingSpaces = [];
		
		//loop through possible spaces
		for (var i in this.possibleSpaces){
			currentSpace = tryflip(this.possibleSpaces[i][0], this.possibleSpaces[i][1], playerTurn);
			
			if (currentSpace[2].length > 0){
				//adds current space to workingSpaces in format (#flipped, x, y, termX, termY)
				this.workingSpaces.push(
					[currentSpace[4],
					this.possibleSpaces[i][0],
					this.possibleSpaces[i][1],
					currentSpace[2]]
					);
			}
		}
		
		//do an end game check
		if (this.workingSpaces.length == 0){
			gameState = 2;
			
			//add up pieces
			for (var x = 0; x < 8; x++){ //loops through first demension
				for (var y = 0; y < 8; y++){
					switch(board[x][y].piece){
						case 1:
							this.score[0] += 1;
							break;
						case 2:
							this.score[1] += 1;
							break;
					}
				}
			}
		}
		
		if (this.debug){
			this.logPossible();
		}
	}
	
	takeTurn(){
		var random;
		
		this.workingSpaces.sort(); //puts the array in order from least flipped to most flipped
		random = Math.floor((Math.random() * this.workingSpaces.length) + 0);
		console.log(this.workingSpaces);
		
		switch(this.difficulty){
			case 1: //flip lowest #flipped value in list (easy)
				flipPieces(
					this.workingSpaces[0][1],
					this.workingSpaces[0][2],
					this.workingSpaces[0][3],
					playerTurn);
				break;
			case 2:
				flipPieces(
					this.workingSpaces[random][1],
					this.workingSpaces[random][2],
					this.workingSpaces[random][3],
					playerTurn);
				break;
			case 3:
				flipPieces(
					this.workingSpaces[this.workingSpaces.length - 1][1],
					this.workingSpaces[this.workingSpaces.length - 1][2],
					this.workingSpaces[this.workingSpaces.length - 1][3],
					playerTurn);
		}
	}
}
/*
Programmer: Zach Hixson
Date: 4/26/2016
Purpose: A General purpose library that will allow the easy building of a UI
*/

class iUI_init { //Main iUI class
	constructor(_canvas){
		this.canvas = _canvas;
		this.ctx = this.canvas.getContext("2d");
		this.bounds = this.canvas.getBoundingClientRect();
		this.frameUpdate = false;
		this.frameRate = 30;
		this.mouseUpdate = true;
		this.mouse = new iUI_Mouse();
		this.scaleRes = 200;
		this.obj = {};
		this.iUIInfo = {
			canvas : this.canvas,
			ctx : this.ctx,
			bounds : this.bounds,
			mouse : this.mouse,
			mouseUpdate : this.mouseUpdate,
			scaleRes : this.scaleRes,
			obj : this.obj,
		};
	}
	
	updateMouse(event){
		this.mouse.x = event.clientX;
		this.mouse.y = event.clientY;
		
		if (this.mouseUpdate){
			this.drawUI();
		}
	}
	
	onMouseDown(){
		this.mouse.mouseDown = true;
		this.drawUI();
	}
	
	onMouseUp(){
		this.mouse.mouseDown = false;
		this.drawUI();
	}
	
	drawUI(){
		for (var i in this.obj){
			this.obj[i].draw();
		}
	}
	
	addBackground(_name, _color){
		if (typeof this.obj[_name] == "undefined"){
			this.obj[_name] = new background(this.iUIInfo, _name, _color);
		}
	}
	
	addText(_name, _text, _size, _x, _y){
		if (typeof this.obj[_name] == "undefined"){
			this.obj[_name] = new text(this.iUIInfo, _name, _text, _size, _x, _y);
		}
	}
	
	addButton(_name, _text, _x, _y, _width, _height){
		if (typeof this.obj[_name] == "undefined"){
			this.obj[_name] = new button(this.iUIInfo, _name, _text, _x, _y, _width, _height);
		}
	}
	
	addGroup(_name){
		if (typeof this.obj[_name] == "undefined"){
			this.obj[_name] = new group(this.iUIInfo, _name);
		}
	}
}

class background{
	constructor(_iUIInfo, _name, _color){
		this.canvas = _iUIInfo.canvas;
		this.ctx = _iUIInfo.ctx;
		this.name = _name;
		this.color = _color;
		this.enabled = true;
	}
	
	draw() {
		if (this.enabled){
			this.ctx.fillStyle = this.color;
			this.ctx.fillRect(0,0, this.canvas.width, this.canvas.height);
		}
	}
}

class text{
	constructor(_iUIInfo, _name, _text, _size, _x, _y){
		this.iUIInfo = _iUIInfo;
		this.canvas = this.iUIInfo.canvas;
		this.ctx = this.iUIInfo.ctx;
		this.bounds = this.iUIInfo.bounds;
		this.scaleRes = this.iUIInfo.scaleRes;
		this.name = _name;
		this.color = "black";
		this.text = _text;
		this.strokeColor = "white";
		this.enabled = true;
		this.font = "Helvetica";
		this.size = _size * (this.canvas.width / this.scaleRes);
		this.unit = "px";
		this.fill = true;
		this.stroke = false;
		this.strokeWidth = 1;
		this.dropShadow = false;
		this.dropShadowSize = 20;
		this.dropShadowColor = "black";
		this.x = _x * (this.canvas.width / this.scaleRes);
		this.y = _y * (this.canvas.height / this.scaleRes);
	}
	
	draw() {
		if (this.enabled){ //check to see if object is enabled
			this.ctx.font = this.size + this.unit + " " + this.font;
			
			if (this.dropShadow){
				this.ctx.shadowBlur = this.dropShadowSize;
				this.ctx.shadowColor = this.dropShadowColor;
			}
			
			if (this.fill){
				this.ctx.fillStyle = this.color;
				this.ctx.fillText(this.text, this.x, this.y);
			}
			
			if (this.stroke){
				this.ctx.strokeStyle = this.strokeColor;
				this.ctx.lineWidth = this.strokeWidth;
				this.ctx.strokeText(this.text, this.x, this.y);
			}
		}
	}
}

class button{
	constructor(_iUIInfo, _name, _text, _x, _y, _width, _height){
		this.iUIInfo = _iUIInfo;
		this.canvas = this.iUIInfo.canvas;
		this.ctx = this.iUIInfo.ctx;
		this.bounds = this.iUIInfo.bounds;
		this.scaleRes = this.iUIInfo.scaleRes;
		this.mouse = this.iUIInfo.mouse;
		this.name = _name;
		this.enabled = true;
		this.text = _text;
		this.x = Math.round(_x * (this.canvas.width / this.scaleRes));
		this.y = Math.round(_y * (this.canvas.height / this.scaleRes));
		this.width = _width * (this.canvas.width / this.scaleRes);;
		this.height = _height * (this.canvas.height / this.scaleRes);;
		this.buttonEnabled = true;
		this.status = 0; //0 = static, 1 = hover, 2 = click;
		this.inactiveFill = "#cccccc";
		this.inactiveBorder = "#bfbfbf";
		this.staticFill = "#f0f0f0";
		this.staticBorder = "#adadad";
		this.hoverFill = "#e5f1fb";
		this.hoverBorder = "#0078d7";
		this.clickFill = "#cce4f7";
		this.clickBorder = "#005499";
	}
	
	drawBackground(_fill, _border){
		this.ctx.fillStyle = _fill;
		this.ctx.strokeStyle = _border;
		this.ctx.fillRect(this.x, this.y, this.width, this.height);
		this.ctx.strokeRect(this.x, this.y, this.width, this.height);
	}
	
	draw(){
		if (this.enabled){
			//draw button background
			if (this.buttonEnabled){
				//update button state
				this.update();
				
				switch(this.status){
					case 0:
						this.drawBackground(this.staticFill, this.staticBorder);
						break;
					case 1:
						this.drawBackground(this.hoverFill, this.hoverBorder);
						break;
					case 2:
						this.drawBackground(this.clickFill, this.clickBorder);
						break;
				}
				
				this.ctx.fillStyle = "black";
			}
			else{
				this.drawBackground(this.inactiveFill, this.inactiveBorder);
				this.ctx.fillStyle = "#adadad";
			}
			
			//draw button text
			this.ctx.font = (this.height * 0.6) + "px Arial";
			this.ctx.fillText(this.text, this.x + (this.width * 0.03), this.y + (this.height * 0.65));
		}
	}
	
	update(){
		if (this.mouse.x >= this.x //+ this.bounds.left
			&& this.mouse.x <= this.x + this.width //+ this.bounds.left
			&& this.mouse.y >= this.y
			&& this.mouse.y <= this.y + this.height //+ this.bounds.top
			){
			
			if (this.mouse.mouseDown){
				this.status = 2;
				
				//execute click code
				this.onClick();
			}
			else{
				this.status = 1;
			}
		}
		else{
			this.status = 0;
		}
	}
	
	onClick(){
		//user will define this section
	}
}

class group{
	constructor(_iUIInfo, _name){
		this.iUIInfo = _iUIInfo;
		this.obj = this.iUIInfo.obj;
		this.name = _name;
		this.items = {};
	}
	
	addItems(){
		var args = arguments;
		
		//add items to list
		for (var i = 0; i < args.length; i++){
			this.items[args[i]] = {};
		}
	}
	
	changeProp(_property, _value){
		//iterate trough items in group and change property (if available)
		for (var i in this.items){
			if (typeof this.obj[i][_property] != "undefined"){
				this.obj[i][_property] = _value;
			}
		}
	}
	
	draw(){
		//every object type needs a draw function, this one is just empty
	}
}

class windowObj{
	constructor(_iUIInfo, _name, _x, _y, _width, _height){
		this.iUIInfo = _iUIInfo;
		this.canvas = this.iUIInfo.canvas;
		this.ctx = this.iUIInfo.ctx;
	}
	
	draw(){
		//
	}
}

class iUI_Mouse{
	constructor(){
		this.x = 0;
		this.y = 0;
		this.mouseDown = false;
	}
}
